import importlib
from django.template.loader import select_template
from django.template.defaultfilters import slugify
from django.template.loader import render_to_string
from django.conf import settings


def get_voucher_forms(prefix=None):
    voucher_forms = {} # Here we store our voucher forms
    for voucher_name, modstring in settings.VOUCHERS:
        modstring = modstring.split('.')
        klass = modstring.pop()
        package = ".".join(modstring)
        module = importlib.import_module(package)
        voucher_form = getattr(module, klass)
        voucher_forms[slugify(voucher_name)] = {'form_class': voucher_form,
                                                'name': voucher_name}
    return voucher_forms


def get_voucher_form(voucher):
    vouchers_forms = get_voucher_forms()
    return vouchers_forms.get(slugify(voucher.voucher), {}).get('form_class')


def get_voucher_form_name(voucher):
    vouchers_forms = get_voucher_forms()
    return vouchers_forms.get(slugify(voucher.voucher), {}).get('name')


def get_voucher_template(voucher=None):
    try_templates = [slugify(voucher.voucher) if voucher else 'voucher',
                     voucher.voucher if voucher else 'voucher',
                     'voucher']
    return select_template(['voucher/%s.html' % voucher for voucher in try_templates])


def render_email_for_voucher_claimed(voucher):
    return render_to_string('notify.html', {'voucher': voucher})



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, render_to_response, redirect
from django.core.urlresolvers import reverse
from django.http import Http404
from django.template import RequestContext
from django.template.defaultfilters import slugify
from django.conf import settings
from django.core.exceptions import ValidationError

from voucher.forms import TokenForm
from voucher.utils import get_voucher_form, get_voucher_form_name, get_voucher_template
from voucher.models import Voucher


LOGIN_URL = getattr(settings, 'LOGIN_URL', '/accounts/login/')


@login_required(login_url=LOGIN_URL)
def claim_voucher(request, human_token=None):
    """
    /vouchers/claim/
    /vouchers/claim/<human_token>
    """
    human_token = slugify(human_token or '')
    voucher_name = ''
    is_claimed = False
    form = None
    voucher = None

    if request.method == 'GET' and human_token:
        voucher = get_object_or_404(Voucher, human_token=human_token)
        form = get_voucher_form(voucher)()
        if voucher.user and voucher.user != request.user:
            raise Http404()
        elif not voucher.user:
            voucher.user = request.user
            voucher.save()
            voucher_name = get_voucher_form_name(voucher)
            messages.success(request, "The voucher %s has been assigned to you, "
                                      "fill in the form to claim your voucher!" % voucher_name)
        is_claimed = voucher.is_claimed()

    elif request.method == 'GET' and not human_token:
        form = TokenForm()
    elif request.method == 'POST':
        token_form = TokenForm(request.POST)
        if token_form.is_valid():
            human_token = slugify(token_form.cleaned_data['token'])
            new_url = reverse('claim_voucher', args=[human_token])
            return redirect(new_url)

        if 'submit' in request.POST and 'Send' in request.POST['submit'] and human_token:
            human_token = slugify(human_token)
            voucher = get_object_or_404(Voucher, human_token=human_token, user=request.user)
            voucher_klass_form = get_voucher_form(voucher)
            voucher_name = get_voucher_form_name(voucher)
            form = voucher_klass_form(request.POST)
            is_claimed = voucher.is_claimed()
            if form.is_valid() and not is_claimed:
                try:
                    voucher.claim_voucher(request.user, human_token, form.cleaned_data)
                    messages.success(request, 'Voucher information has been sent!')
                except ValidationError as error:
                    messages.error(request, '\n'.join(error.messages))

    template = get_voucher_template(voucher)
    return render_to_response(template.name,
                              RequestContext(request, {'voucher_name': voucher_name,
                                                       'token': human_token,
                                                       'is_claimed': is_claimed,
                                                       'form': form,
                                                       'voucher': voucher}))



import datetime
from jsonfield.fields import JSONField
from django.template.defaultfilters import slugify
from django.core.exceptions import ValidationError
from django.core.mail import EmailMultiAlternatives
from django.conf import settings
from django.shortcuts import get_object_or_404
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy
from django.db import models

from voucher.humanhash import HumanHasher

from voucher.utils import get_voucher_forms, render_email_for_voucher_claimed, get_voucher_form


VOUCHER_SEND_NOTIFICATION = getattr(settings, 'VOUCHER_SEND_NOTIFICATION', True)
NOTIFY_VOUCHER_FROM = getattr(settings, 'NOTIFY_VOUCHER_FROM', None)
VOUCHER_MANAGERS = getattr(settings, 'VOUCHER_MANAGERS', [])

VOUCHERS = get_voucher_forms()


def get_voucher_choices():
    return [(s_name, voucher['name']) for s_name, voucher in VOUCHERS.items()]


class Voucher(models.Model):
    creation_date = models.DateTimeField(auto_now_add=True, verbose_name=ugettext_lazy('Creation Date'))
    claimed_date = models.DateTimeField(verbose_name=ugettext_lazy('Used Date'), blank=True, null=True)
    expiry_date = models.DateTimeField(verbose_name=ugettext_lazy('Expiration Date'), blank=True, null=True)
    voucher = models.CharField(choices=get_voucher_choices(), max_length=50)
    voucher_info = JSONField(blank=True, null=True)
    token = models.CharField(unique=True, db_index=True, max_length=50)
    human_token = models.CharField(unique=True, db_index=True, max_length=255)
    user = models.ForeignKey(User, blank=True, null=True, related_name='vouchers')
    created_by = models.ForeignKey(User, verbose_name=ugettext_lazy('Created By'), blank=True, null=True,
                                   related_name='my_vouchers')
    last_edit_datetime = models.DateTimeField(auto_now=True, blank=True, null=True)
    last_editor = models.ForeignKey(User, verbose_name=ugettext_lazy('Last Editor'), blank=True, null=True,
                                   related_name='edited_vouchers')
    notified = models.BooleanField(default=False)
    activated = models.BooleanField(default=True)

    def generate_token(self):
        human_hasher = HumanHasher()
        return human_hasher.uuid()

    def reactivate_voucher(self):
        self.human_token, self.token = self.generate_token()
        self.activated = True
        self.claimed_date = None
        self.notified = False
        return self.human_token, self.token

    def is_valid_token(self, token):
        human_hasher = HumanHasher()
        return human_hasher.humanize(token) == self.human_token

    def is_claimed(self):
        return bool(self.user and self.claimed_date and not self.activated and self.voucher)

    def assign_voucher(self, user):
        if self.user:
            raise ValidationError("Cannot assign voucher, already taken.")

        if not self.is_claimed():
            self.user = user
            self.save()

    def claim_voucher(self, user, human_token, voucher_info=None):
        if self.is_claimed():
            raise ValidationError("Voucher already claimed by %s" % self.user)

        if not slugify(self.voucher) in VOUCHERS:
            raise ValidationError("Invalid voucher choice.")

        voucher_form = get_voucher_form(self)(voucher_info or {})
        if not voucher_form.is_valid():
            raise ValidationError("Invalid voucher info.")

        if human_token != self.human_token:
            raise ValidationError("Cannot claim voucher, invalid token.")

        if not isinstance(user, User):
            user = get_object_or_404(User, username=user)

        if not self.user:
            self.user = user
        elif self.user and user != self.user:
            raise ValidationError("Cannot claim voucher, the user assigned don't match")

        self.voucher_info = voucher_form.cleaned_data
        self.claimed_date = datetime.datetime.now()
        self.activated = False
        self.notified = False
        self.save()

    def get_voucher_form(self, voucher_info=None):
        voucher_info = voucher_info or self.voucher_info or {}
        if self.voucher and self.voucher in VOUCHERS:
            form = get_voucher_form(self)(voucher_info)
            form.is_valid()
            return form

    def get_voucher_info_template(self):
        if self.voucher_info and self.voucher:
            voucher_form = get_voucher_form(self)
            for field in voucher_form.base_fields:
                voucher_form.base_fields[field].widget.attrs['readonly'] = True
            voucher_form = voucher_form(self.voucher_info)
            if voucher_form.is_valid():
                return voucher_form.as_table()
        return ''

    def save(self, *args, **kwargs):
        self.voucher = slugify(self.voucher)
        if not self.voucher in VOUCHERS:
            raise ValueError("Cannot create voucher, invalid voucher choice... (%s)" % self.voucher)
        if not self.token:
            for n_try in range(11):
                if n_try == 10:
                    raise ValidationError("Cannot create voucher, max attempts reached.")
                human_token, token = self.generate_token()
                try:
                    Voucher.objects.get(human_token=human_token, token=token)
                    Voucher.objects.create()
                except Voucher.DoesNotExist:
                    self.human_token = human_token
                    self.token = token
                    break
        elif VOUCHER_SEND_NOTIFICATION and self.is_claimed() and not self.notified:
            if not NOTIFY_VOUCHER_FROM:
                raise ValidationError("You must define a sender in your settings.py NOTIFY_VOUCHER_FROM")
            self.notified = True
            subject = "Voucher claimed (%s) by %s" % (self.voucher, self.user)
            from_email, to = NOTIFY_VOUCHER_FROM, [i for (_, i) in VOUCHER_MANAGERS]
            html_content = render_email_for_voucher_claimed(self)
            msg = EmailMultiAlternatives(subject, subject, from_email, to)
            msg.attach_alternative(html_content, "text/html")
            msg.send(fail_silently=False)
        super(Voucher, self).save(*args, **kwargs)



from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('',
    #url(r'^$', VoucherList.as_view(), name="voucher_list"),
    url(r'claim/$', 'voucher.views.claim_voucher', name='claim_voucher'),
    url(r'claim/(?P<human_token>[-\w]+)$', 'voucher.views.claim_voucher', name='claim_voucher'),
)



import datetime

from django.contrib import admin
from voucher.forms import VoucherAdminForm
from voucher.models import Voucher


def generate_token(self, request, queryset):
    for voucher in queryset.all():
        voucher.generate_token()


def deactivate_token(self, request, queryset):
    for voucher in queryset.all():
        voucher.activated = False
        voucher.save()


def reactivate_token(self, request, queryset):
    max_tries = 5
    for voucher in queryset.all():
        tries = 0
        while tries < max_tries:
            voucher.reactivate_voucher()
            try:
                obj = Voucher.objects.get(human_token=voucher.human_token)
                if voucher.pk != obj.pk:
                    tries += 1
                else:
                    break
            except Voucher.MultipleObjectsReturned:
                # This never should happen since there is a Unique constraint for this field, but lets be more safety.
                tries += 1
            except Voucher.DoesNotExist:
                break
        voucher.save()


class VoucherAdmin(admin.ModelAdmin):
    raw_id_fields = ['user']
    list_display = ['voucher', 'user', 'activated', 'notified', 'claimed_date',
                    'created_by', 'last_editor', 'human_token']
    readonly_fields = ['creation_date', 'claimed_date', 'activated', 'voucher_info', 'created_by', 'last_editor',
                       'last_edit_datetime', 'notified']
    actions = [generate_token, deactivate_token, reactivate_token]
    form = VoucherAdminForm
    fieldsets = (('Voucher Info', {'fields': ['voucher', 'token', 'human_token', 'expiry_date', 'user', 'claimed_date']}),
                 ('Edition', {'fields': ['created_by', 'creation_date', 'last_editor', 'last_edit_datetime']}),
                 ('Status', {'fields': ['notified', 'activated']}))

    def get_form(self, request, obj=None, **kwargs):
        form = super(VoucherAdmin, self).get_form(request, obj=obj, **kwargs)
        if obj:
            obj.last_editor = request.user
            obj.save()
        if not obj:
            form.creator = request.user
        return form

    def save_model(self, request, obj, form, change):
        if obj and not obj.pk:
            obj.created_by = request.user

        obj.last_editor = request.user
        obj.last_edit_datetime = datetime.datetime.now()

        super(VoucherAdmin, self).save_model(request, obj, form, change)


admin.site.register(Voucher, VoucherAdmin)



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='django-vouchers',
    version="0.5",
    author='Federico Castro',
    author_email='fmc0208@gmail.com',
    description='Vouchers in Django',
    url='https://github.com/yeago/django-vouchers',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development"
    ],
)



from django import forms
from voucher.humanhash import HumanHasher

from voucher.models import Voucher


class VoucherAdminForm(forms.ModelForm):
    class Meta:
        model = Voucher

    def __init__(self, *args, **kwargs):
        super(VoucherAdminForm, self).__init__(*args, **kwargs)
        token, human_token = self.generate_token()
        self.fields['token'].initial = token
        self.fields['token'].widget.attrs['readonly'] = True

        self.fields['human_token'].initial = human_token
        self.fields['human_token'].widget.attrs['readonly'] = True

    def generate_token(self):
        max_tries, tries = 5, 0
        while tries <= max_tries:
            human_hasher = HumanHasher()
            human_token, token = human_hasher.uuid()
            try:
                Voucher.objects.get(human_token=human_token)
                tries += 1
            except Voucher.DoesNotExist:
                break
        return token, human_token

    def refresh_tokens(self):
        token, human_token = self.generate_token()
        self.cleaned_data['token'] = token
        self.cleaned_data['human_token'] = human_token


class TokenForm(forms.Form):
    token = forms.CharField(max_length=100, widget=forms.TextInput(attrs={'class': 'form-control'}))
